import scala.swing._
import scala.swing.event.ButtonClicked

/**
 * Számológép program
 *
 * @author Kovács Vince <kovqaai@inf.elte.hu>
 */
class UI extends MainFrame {
  // Keret címe
  title = "Calculator"
  // Keret tartalma
  contents = new GridBagPanel {

    // Aktuális értek (számolt érték)
    var actualValue = 0.0

    // Legutolsó érték (bevitt érték)
    var lastValue = 0.0

    // Aktuális művelet
    var operation = ""

    // Tárolja, hogy felülírható-e a számlap
    var reset = true

    // Egyenlőség leütését tárolja
    var btnEqualsPressed = false

    // Számlap
    val display = new TextField() {
      // Beállítja a kezdőértéket a szövegmezőnek
      text = "0"
      // Letiltja, hogy a szövegmező szerkeszthető legyen
      editable = false
      // Szöveg méretének beállítása
      font = new Font(font.getFontName, java.awt.Font.BOLD, 20)
      // Szövegmező méretének beállítása
      preferredSize = new Dimension(60, 50)
      // Beállítja, hogy a szöveg jobbra legyen igazítva
      horizontalAlignment = Alignment.Right
    }

    // Gombok létrehozása
    val btn0 = createBtn("0")
    val btn1 = createBtn("1")
    val btn2 = createBtn("2")
    val btn3 = createBtn("3")
    val btn4 = createBtn("4")
    val btn5 = createBtn("5")
    val btn6 = createBtn("6")
    val btn7 = createBtn("7")
    val btn8 = createBtn("8")
    val btn9 = createBtn("9")

    val btnBack = createBtn("←")
    val btnCE = createBtn("CE")
    val btnC = createBtn("C")
    val btnPlusMinus = createBtn("±")
    val btnSquareRoot = createBtn("√")
    val btnReciProc = createBtn("1/x")

    val btnPercentage = createBtn("%")
    val btnDivision = createBtn("÷")
    val btnMultiplication = createBtn("×")
    val btnSummation = createBtn("+")
    val btnSubstraction = createBtn("-")
    val btnEquality = createBtn("=")
    val btnDash = createBtn(",")

    // Gombok regisztrálása az eseményvezérlőhöz
    listenTo(btn0, btn1, btn2, btn3, btn4,
             btn5, btn6, btn7, btn8, btn9,
             btnBack, btnCE, btnC, btnPlusMinus, btnSquareRoot, btnReciProc,
             btnDivision, btnMultiplication, btnSummation, btnSubstraction,
             btnPercentage, btnEquality, btnDash
            )

    // Események megírása a regisztrált gombokhoz
    reactions += {
      case ButtonClicked(`btn0`) =>
        // Hozzáad egy 0 a számlaphoz
        addCharToDisplay('0')
      case ButtonClicked(`btn1`) =>
        // Hozzáad egy 1 a számlaphoz
        addCharToDisplay('1')
      case ButtonClicked(`btn2`) =>
        // Hozzáad egy 2 a számlaphoz
        addCharToDisplay('2')
      case ButtonClicked(`btn3`) =>
        // Hozzáad egy 3 a számlaphoz
        addCharToDisplay('3')
      case ButtonClicked(`btn4`) =>
        // Hozzáad egy 4 a számlaphoz
        addCharToDisplay('4')
      case ButtonClicked(`btn5`) =>
        // Hozzáad egy 5 a számlaphoz
        addCharToDisplay('5')
      case ButtonClicked(`btn6`) =>
        // Hozzáad egy 6 a számlaphoz
        addCharToDisplay('6')
      case ButtonClicked(`btn7`) =>
        // Hozzáad egy 7 a számlaphoz
        addCharToDisplay('7')
      case ButtonClicked(`btn8`) =>
        // Hozzáad egy 8 a számlaphoz
        addCharToDisplay('8')
      case ButtonClicked(`btn9`) =>
        // Hozzáad egy 9 a számlaphoz
        addCharToDisplay('9')
      case ButtonClicked(`btnDash`) =>
        // Hozzáad egy . a számlaphoz
        addCharToDisplay('.')
      case ButtonClicked(`btnBack`) =>
        // Töröljük az utolsó karaktert
        removeLastCharFromDisplay()
      case ButtonClicked(`btnCE`) =>
        // Kinullázza a számlapot
        display.text = "0"
        // Beállítja, hogy nem egyenlőség volt megnyomva
        btnEqualsPressed = false
      case ButtonClicked(`btnC`) =>
        // Kinullázza a számlapot
        display.text = "0"
        // Kinullázza az aktuális állapotot
        actualValue = 0
        // Alapértelmezettre állítja a műveletet
        operation = ""
        // Beállítja, hogy nem egyenlőség volt megnyomva
        btnEqualsPressed = false
      case ButtonClicked(`btnPlusMinus`) =>
        // Számlap felülírása egy feltételes vezérlési szerkezetbe
        display.text = if (display.text.head == '-') {
          // Levágja a szöveg elejét
          display.text.substring(1)
        }
        else if (!display.text.equals("0")) {
          // Hozzáad egy '-' jelet a szöveg elejére
          '-' + display.text
        }
        else {
          // Visszaadjuk önmagát, hogy vezérlés teljes legyen
          display.text
        }
        // Beállítja, hogy nem egyenlőség volt megnyomva
        btnEqualsPressed = false
      case ButtonClicked(`btnSquareRoot`) =>
        // Kijelzőn található érték eltárolása
        lastValue = toDouble(display.text)
        // Kiszámolja az aktuális értéket
        calcActual()
        // Beállítja a műveletet
        operation = "√"
        // Végrehajta a műveletet azonnal
        calcActual()
        // Beállítja, hogy nem egyenlőség volt megnyomva
        btnEqualsPressed = false
      case ButtonClicked(`btnReciProc`) =>
        // Kijelzőn található érték eltárolása
        lastValue = toDouble(display.text)
        // Kiszámolja az aktuális értéket
        calcActual()
        // Beállítja a műveletet
        operation = "1/x"
        // Végrehajtja a műveletet azonnal
        calcActual()
        // Beállítja, hogy nem egyenlőség volt megnyomva
        btnEqualsPressed = false
      case ButtonClicked(`btnDivision`) =>
        // Kijelzőn található érték eltárolása
        lastValue = toDouble(display.text)
        // Kiszámolja az aktuális értéket
        calcActual()
        // Beállítja a műveletet
        operation = "/"
        // Beállítja, hogy nem egyenlőség volt megnyomva
        btnEqualsPressed = false
      case ButtonClicked(`btnMultiplication`) =>
        // Kijelzőn található érték eltárolása
        lastValue = toDouble(display.text)
        // Kiszámolja az aktuális értéket
        calcActual()
        // Beállítja a műveletet
        operation = "*"
        // Beállítja, hogy nem egyenlőség volt megnyomva
        btnEqualsPressed = false
      case ButtonClicked(`btnSummation`) =>
        // Kijelzőn található érték eltárolása
        lastValue = toDouble(display.text)
        // Kiszámolja az aktuális értéket
        calcActual()
        // Beállítja a műveletet
        operation = "+"
        // Beállítja, hogy nem egyenlőség volt megnyomva
        btnEqualsPressed = false
      case ButtonClicked(`btnSubstraction`) =>
        // Kijelzőn található érték eltárolása
        lastValue = toDouble(display.text)
        // Kiszámolja az aktuális értéket
        calcActual()
        // Beállítja a műveletet
        operation = "-"
        // Beállítja, hogy nem egyenlőség volt megnyomva
        btnEqualsPressed = false
      case ButtonClicked(`btnEquality`) =>
        // Leellenőrzi, hogy legutoljára egyenlőség volt-e lenyomva
        if (!btnEqualsPressed) {
          // Kijelzőn található érték eltárolása
          lastValue = toDouble(display.text)
        }
        // Kiszámolja az aktuális értéket
        calcActual()
        // Beállítja, hogy egyenlőség volt megnyomva
        btnEqualsPressed = true
    }

    /**
     * Szöveget számmá alakítja át
     *
     * @param s szöveg
     * @return átalakított szám
     */
    def toDouble(s: String): Double = {
      try {
        // Megpróbálja számmá alakítani a szöveget
        s.toDouble
      } catch {
        // Hiba esetén 0-t ad vissza
        case e: Exception => 0
      }
    }

    /**
     * Képernyőhöz hozzáad egy a karaktert
     *
     * @param c karakter
     */
    def addCharToDisplay(c: Char) {
      // Leellenőrzi, hogy legutoljára egyenlőség volt-e lenyomva
      if (btnEqualsPressed) {
        // Beállítja az aktuális értéket alapértelmezettre
        actualValue = 0
        // Beállítja az utolsó értéket alapértelmezte
        lastValue = 0
        // Beállítja a művelet alapértelmezettre
        operation = ""
        // Beállítja, hogy nem egyenlőség volt megnyomva
        btnEqualsPressed = false
      }
      // Leellenőrizzük, hogy a szöveg "0"-e, vagy törölhető-e a számlap
      if (display.text.equals("0") || reset) {
        // Töröljük a számlapot
        display.text = ""
      }

      // Beállítjuk, hogy a számlapot a következő karakter beírásakor ne töröljük
      reset = false
      // Hozzáadjuk a számlap végéhez a paraméterben átadott karaktert
      display.text += c
    }

    /**
     * A számlapról az utolsó karaktert törli
     */
    def removeLastCharFromDisplay() {
      // Megvizsgálja, hogy a számlap törölhető-e
      if (reset) {
        // Számlap törlése
        display.text = ""
        // Beállítja, hogy a számlapot a következő karakter beírásakor ne töröljük
        reset = false
      }
      // Töröli az utoljára hozzáadott karakter
      display.text = display.text.dropRight(1)

      // Leellenőrizi, hogy üres-e a számlap
      if (display.text.isEmpty) {
        // Alapértelmezett értékre állítja a számlapot
        display.text = "0"
      }
      // Beállítja, hogy nem egyenlőség volt megnyomva
      btnEqualsPressed = false
    }

    /**
     * Kiszámolja és eltárolja az aktuális értéket
     */
    def calcActual() {
      // Értekedás egy feltételes vezérlési szerkezetben
      actualValue = operation match {
        // Ha nincs megadva művelet
        case "" =>
          lastValue
        // Ha a gyökvonás művelet van megadva
        case "√" =>
          math.sqrt(actualValue)
        // Ha a reciprok művelet van megadva
        case "1/x" =>
          1 / actualValue
        // Ha a szorzás művelet van megadva
        case "*" =>
          actualValue * lastValue
        // Ha a osztás művelet van megadva
        case "/" =>
          actualValue / lastValue
        // Ha a összeadás művelet van megadva
        case "+" =>
          actualValue + lastValue
        // Ha a kivonás művelet van megadva
        case "-" =>
          actualValue - lastValue
      }

      // Beállítjuk, hogy a számlapot felül lehessen írni a következő bevitelnél
      reset = true
      // Kiírja az aktuális állapotot a számlapra
      display.text = actualValue.toString
    }


    /**
     * Gomb létrehozása a GUI-n
     *
     * @param text gombon megjelenő szöveg
     * @return     gomb objektum
     */
    def createBtn(text: String): Button = {
      // Gomb objektum létrehozása
      val b = new Button
      // Gomb szövegének beállítása
      b.text = text
      // Szöveg méretének beállítása
      b.font = new Font(b.font.getFontName, b.font.getStyle, 15)
      // Gomb méretének beállítása
      b.preferredSize = new Dimension(60, 50)
      // Gomb visszaadása
      b
    }

    /**
     *
     * Elem elhelyezése a megjelenítő rácsba
     *
     * @param x          bal felső saroktól vett X koordináta
     * @param y          bal felső saroktól vett Y koordináta
     * @param gridWidth  rács szélessége
     * @param gridHeight rács magassága
     * @param weightX    mennyire kerüljön a cella átméretezésre vízszintes irányba, ha az ablak átméreteződik
     * @param weightY    mennyire kerüljön a cella átméretezésre függőleges irányba, ha az ablak átméreteződik
     * @param fill       kitöltés
     * @return
     */
    def constraints(x: Int, y: Int,
                    gridWidth: Int = 1, gridHeight: Int = 1,
                    weightX: Double = 0.0, weightY: Double = 0.0,
                    fill: GridBagPanel.Fill.Value = GridBagPanel.Fill.Both): Constraints = {
      // Létrehozunk egy új objektumot
      val c = new Constraints

      // Rács X pozíciójának beállítása
      c.gridx = x
      // Rács Y pozíciójának beállítása
      c.gridy = y
      // Rács szélességének beállítása
      c.gridwidth = gridWidth
      // Rács magasságának beállítása
      c.gridheight = gridHeight
      // Az X átméretezési szorzó beállítása
      c.weightx = weightX
      // Az Y átméretezési szorzó beállítása
      c.weighty = weightY
      // Kitöltés típusának beállítása
      c.fill = fill
      // Visszaadjuk az objektumot
      c
    }

    // Az 1. sor létrehozása
    add(display, constraints(0, 0, gridWidth = 5)) // 5 rácsméretet függőlegesen kitöltő szövegmező

    // A 2. sor létrehozása
    add(btnBack, constraints(0, 1))
    add(btnCE, constraints(1, 1))
    add(btnC, constraints(2, 1))
    add(btnPlusMinus, constraints(3, 1))
    add(btnSquareRoot, constraints(4, 1))

    // A 3. sor létrehozása
    add(btn7, constraints(0, 2))
    add(btn8, constraints(1, 2))
    add(btn9, constraints(2, 2))
    add(btnDivision, constraints(3, 2))
    add(btnPercentage, constraints(4, 2))

    // A 4. sor létrehozása
    add(btn4, constraints(0, 3))
    add(btn5, constraints(1, 3))
    add(btn6, constraints(2, 3))
    add(btnMultiplication, constraints(3, 3))
    add(btnReciProc, constraints(4, 3))

    // Az 5. sor létrehozása
    add(btn1, constraints(0, 4))
    add(btn2, constraints(1, 4))
    add(btn3, constraints(2, 4))
    add(btnSubstraction, constraints(3, 4))
    add(btnEquality, constraints(4, 4, gridHeight = 2)) // 2 rácsméretet függőlegesen kitöltő gomb

    // A 6. sor létrehozása
    add(btn0, constraints(0, 5, gridWidth = 2)) // 2 rácsméretet vízszintesen kitöltő gomb
    add(btnDash, constraints(2, 5))
    add(btnSummation, constraints(3, 5))
  }

}

/**
 * SimpleSwingApllication osztály tartalmaz már egy main függvényt,
 * tehát ebben az esetben az hívódik meg
 */
object Calculator extends SimpleSwingApplication {
  def top = new UI
}
